function PhotonicCrystal(Cell,Sphere,Calc)
if nargin < 1
    Cell.Bravais = 'fcc';
    Cell.e = 13; % e/e0 >= 1
    Cell.a = 100; % [nm] conventional unit cell
    Cell.b = 80; % [nm] conventional unit cell
    Cell.c = 140; % [nm] conventional unit cell
    Cell.alpha = 70; % [deg] conventional unit cell
    Cell.beta =  60; % [deg] conventional unit cell
    Cell.gamma = 110; % [deg] conventional unit cell
end
if nargin < 2
    Sphere(1).e = 1; % material e/e0 >= 1(vacuum)
    Sphere(1).Radius = 2/3*1/(2*sqrt(2)); % [a] of  conventional unit cell
    Sphere(1).M = [.5 .5 .5]; % [a,b,c] of conventional unit cell
end
if nargin < 3
    Calc.naMax = 50; % NPp max UnitCellSide NVoxels <= naMax^3!
    Calc.nDOS = 50; % NPp reziprocal UnitCellSide for DOS, NVoxels = nDOS^3!
    Calc.nGMax = 15; % N G-vectors for calculation of w's size Mat = 2N x 2N
    Calc.nDispersion = 250; % n w-values Dispersion relation
    Calc.yLimDispersion = [];
    Calc.EmptyLatticeApproximation = 0;
    Calc.plotReal = true; % Real space Unit Cell
    Calc.plotFour = true; % Reciprocal Space Unit Cell
    Calc.plotDispersion = true; % Dispersion Relation
    Calc.plotDOS = true; % Density of States
    Calc.saveplots = false; % automatic figure saving
    Calc.save = false; % automatic set saving
    Calc.name = 'noname';
    Calc.mfilename = [];
    Calc.fG = 'fftn'; % 'FFTN', 'NUM', 'ANA'
end



%% Definitions: numeric approximation
Real.naMax = Calc.naMax; % n points per UnitCellSide number of voxels <= Real.naMax^3!
Four.nGMax = Calc.nGMax; % n G-vectors for calculation of w's (size of square matrice)
Four.nDispersion = Calc.nDispersion; % n EigProblems Dispersion Relation
Four.nDOS = Calc.nDOS; % n EigProblems DOS

%% Definitions: plotting
P.rectl = [300,100,900,700];
P.rectp = [300,100,800,900];
P.CBG = [0 0 0];
P.CAX = [.7 .7 .7];
P. SavePathFigure = ['.\figures','\',Calc.name];
P.fmt = '%22s';

%% start
% clc
fprintf('<strong>calculating %s</strong>\n',Calc.name)
addpath '.\functions';
if Calc.save
    newbackup=[P.SavePathFigure,'\',Calc.mfilename(find(Calc.mfilename=='\',1,'last')+1:end),'.m'];
    currentfile=strcat(Calc.mfilename,'.m');
    if ~isdir(P.SavePathFigure)
        mkdir(P.SavePathFigure)
    end
    copyfile(currentfile,newbackup);
    save([P.SavePathFigure,'\settings.mat'],'Cell','Sphere','Calc')
end
if strcmp(getenv('username'),'bdumboeck')
    Calc.SendMail = true;
else
    Calc.SendMail = false;
end

%% Bravais lattice characteristics
switch strrep(lower(Cell.Bravais),' ','')
    case {'triclinic'} % #1
        Cell.Bravais = 'triclinic';
        % a ~= b ~= c
        % alpha ~= beta ~= gamma
        Cell.Type = 'simple';
        Four.Trace = 'XGUGZGR';
        Four.Points = struct(...
            'R',[.50 .50 .50],...
            'U',[.50 .00 .50],...
            'X',[.50 .00 .00],...
            'Z',[.00 .00 .50]);
    case {'monoclinic','simplemonoclinic'} % #2
        Cell.Bravais = 'monoclinic'; 
        % a ~= b ~= c
        % alpha = gamma = 90
        % beta ~= 90
        Cell.alpha = 90;
        Cell.gamma = 90;
        Cell.Type = 'simple';
        Four.Trace = 'ABGDEYCGZ';
        Four.Points = struct(...
            'A',[.50 .50 .00],...
            'B',[.50 .00 .00],...
            'C',[.00 .50 .50],...
            'D',[.50 .00 .50],...
            'E',[.50 .50 .50],...
            'Y',[.00 .50 .00],...
            'Z',[.00 .00 .50]);
    case {'basecenteredmonoclinic'} % #3
        Cell.Bravais = 'base centered monoclinic';
        % a ~= b ~= c
        % alpha = beta = 90
        % gamma ~= 90
        Cell.alpha = 90;
        Cell.beta = 90;
        Cell.Type = 'base';
        Four.Trace = 'LGAMYG';
        Four.Points = struct(...
            'A',[.50 .00 .00],...
            'L',[.50 .50 .00],...
            'M',[.50 .00 .50],...
            'Y',[.00 .00 .50]);
    case {'orthorhombic','simpleorthorhombic'} % #4
        Cell.Bravais = 'orthorhombic';
        % a ~= b ~= c
        % alpha = beta = gamma = 90
        Cell.alpha = 90;
        Cell.beta = 90;
        Cell.gamma = 90;
        Cell.Type = 'simple';
        Four.Trace = 'XGYSRUZGTY';
        Four.Points = struct(...
            'R',[.50 .50 .50],...
            'S',[.50 .50 .00],...
            'T',[.00 .50 .50],...
            'U',[.50 .00 .50],...
            'X',[.50 .00 .00],...
            'Y',[.00 .50 .00],...
            'Z',[.00 .00 .50]);
    case {'basecenteredorthorhombic'} % #5
        Cell.Bravais = 'base centered orthorhombic';
        % a ~= b ~= c
        % alpha = beta = gamma = 90
        Cell.alpha = 90;
        Cell.beta = 90;
        Cell.gamma = 90;
        Cell.Type = 'base';
        Four.Trace = 'RSGZTYGR';
        Four.Points = struct(...
            'R',[.50 .00 .50],...
            'S',[.50 .00 .00],...
            'T',[.50 .50 .50],...
            'Y',[.50 .50 .00],...
            'Z',[.00 .00 .50]);
	case {'facecenteredorthorhombic'} % #6
        Cell.Bravais = 'face centered orthorhombic';
        % a ~= b ~= c
        % alpha = beta = gamma = 90
        Cell.alpha = 90;
        Cell.beta = 90;
        Cell.gamma = 90;
        Cell.Type = 'face';
        Four.Trace = 'LGTGYGZ';
        Four.Points = struct(...
            'L',[.50 .50 .50],...
            'T',[.00 .50 .50],...
            'Y',[.00 .50 .00],...
            'Z',[.00 .00 .50]);
    case {'bodycenteredorthorhombic'} % #7
        Cell.Bravais = 'body centered orthorhombic';
        % a ~= b ~= c
        % alpha = beta = gamma = 90
        Cell.alpha = 90;
        Cell.beta = 90;
        Cell.gamma = 90;
        Cell.Type = 'body';
        Four.Trace = 'RGSGWGX';
        Four.Points = struct(...
            'R',[.50 .00 .50],...
            'S',[.00 .50 .50],...
            'W',[.50 .50 .50],...
            'X',[.50 .00 .00]);
    case {'tetragonal','simpletetragonal'} % # 8
        Cell.Bravais = 'tetragonal';
        % a ~= b ~= c
        % alpha = beta = gamma = 90
        Cell.alpha = 90;
        Cell.beta = 90;
        Cell.gamma = 90;
        Cell.Type = 'simple';
        Four.Trace = 'MGXARGZ'; %
        Four.Points = struct(...
            'A',[.50 .50 .50],...
            'M',[.50 .50 .00],...
            'R',[.00 .50 .50],...
            'X',[.00 .50 .00],...
            'Y',[.50 .00 .00],...
            'Z',[.00 .00 .50]);
    case {'bodycenteredtetragonal'} % # 9
        Cell.Bravais = 'body centered tetragonal';
        % a ~= b ~= c
        % alpha = beta = gamma = 90
        Cell.alpha = 90;
        Cell.beta = 90;
        Cell.gamma = 90;
        Cell.Type = 'body';
        Four.Trace = 'XGPNGXP'; %
        Four.Points = struct(...
            'N',[.00 .50 .00],...
            'P',[.50 .50 .50],...
            'X',[.50 .00 .00],...
            'Z',[.00 .00 .50]);
    case {'sc','cubic','simplecubic'} % # 10
        Cell.Bravais = 'simple cubic';
        % a = b = c
        % alpha = beta = gamma = 90
        Cell.b = Cell.a;
        Cell.c = Cell.a;
        Cell.alpha = 90;
        Cell.beta = 90;
        Cell.gamma = 90;
        Cell.Type = 'simple';
        Four.Trace = 'XGMRXGR'; %
        Four.Points = struct(...
            'M',[.50 .50 .00],...
            'R',[.50 .50 .50],...
            'X',[.00 .50 .00]);
    case {'fcc' 'facecenteredcubic'} % # 11
        Cell.Bravais = 'face centered cubic';
        % a = b = c
        % alpha = beta = gamma = 90
        Cell.b = Cell.a;
        Cell.c = Cell.a;
        Cell.alpha = 90;
        Cell.beta = 90;
        Cell.gamma = 90;
        Cell.Type = 'face';
        Four.Trace = 'XULGXWK'; %
        Four.Points = struct(...
            'K',[3/8 3/4 3/8],...
            'L',[1/2 1/2 1/2],...
            'U',[1/4 5/8 5/8],...
            'W',[1/4 3/4 1/2],...
            'X',[.00 1/2 1/2]);
    case {'bcc','bodycenteredcubic'} % # 12
        Cell.Bravais = 'body centered cubic';
        % a = b = c
        % alpha = beta = gamma = 90
        Cell.b = Cell.a;
        Cell.c = Cell.a;
        Cell.alpha = 90;
        Cell.beta = 90;
        Cell.gamma = 90;
        Cell.Type = 'body';
        Four.Trace = 'NHGPNGHP';
        Four.Points = struct(...
            'H',[.00 1.0 .50],...
            'N',[.00 .50 .00],...
            'P',[.50 .50 .25]);
    case {'rhombohedral','trigonal'} % # 13
        Cell.Bravais = 'rhombohedral';
        % a = b = c
        % alpha = beta = gamma < 120
        Cell.b = Cell.a;
        Cell.c = Cell.a;
        Cell.beta = Cell.alpha;
        Cell.gamma = Cell.alpha;
        Cell.Type = 'simple';
        Four.Trace = 'LGTGZ';
        Four.Points = struct(...
            'L',[.50 .00 .00],...
            'T',[.50 .50 .00],...
            'Z',[.00 .00 .50]);
    case {'hex','hexagonal'} % # 14
        Cell.Bravais = 'hexagonal';
        % a = b ~= c
        % alpha = 120
        % beta = gamma = 90
        Cell.b = Cell.a;
        Cell.alpha = 90;
        Cell.beta = 90;
        Cell.gamma = 120;
        Cell.Type = 'simple';
        Four.Trace = 'KGAHMGL';
        Four.Points = struct(...
            'A',[.00 .00 .50],...
            'H',[1/3 1/3 .50],...
            'K',[1/3 1/3 .00],...
            'L',[.50 .00 .50],...
            'M',[.50 .00 .00]);
    otherwise
        error(['Cell.Bravais Lattice "',Cell.Bravais,'" not supported'])
end
Four.Points.G = [0 0 0];
Cell.a = 1e-9*Cell.a; % [m]
Cell.b = 1e-9*Cell.b; % [m]
Cell.c = 1e-9*Cell.c; % [m]

%% Real space: Cell, Prim, Real
[Real,Prim,Cell] = BasisVectors(Real,Cell);

aMax = max([Prim.a,Prim.b,Prim.c]);
na1 = round(Real.naMax*Prim.a/aMax);
na2 = round(Real.naMax*Prim.b/aMax);
na3 = round(Real.naMax*Prim.c/aMax);
Real.na1 = na1;
Real.na2 = na2;
Real.na3 = na3;
clear aMax

Cell.Volume = dot(Cell.a1,cross(Cell.a2,Cell.a3));
Prim.Volume = dot(Prim.a1,cross(Prim.a2,Prim.a3));

[Real.xx,Real.yy,Real.zz] = Space(Prim.a1,Prim.a2,Prim.a3,...
    linspace(.5/na1,1-.5/na1,na1),linspace(.5/na2,1-.5/na2,na2),linspace(.5/na3,1-.5/na3,na3));
Real.A = Prim.e*ones(na1,na2,na3);
clear na1 na2 na3

%% Real Space: Spheres
if Calc.EmptyLatticeApproximation
    Sphere(:) = [];
    Real.A = 1*Real.A;
else
    [xx,yy,zz] = Space(Prim.a1,Prim.a2,Prim.a3,-1:1);
    for k = 1:numel(Sphere)
        Sphere(k).M = Sphere(k).M*[Cell.a1;Cell.a2;Cell.a3];
        Sphere(k).Radius = Cell.a*Sphere(k).Radius;
        for j = 1:numel(xx)
            Real.A(getR2(Real,Sphere(k).M+[xx(j) yy(j) zz(j)])<Sphere(k).Radius^2) = Sphere(k).e; % e/e0
        end
    end
    clear xx yy zz
end

%% Reziprocal space: Four
Four.g1 = Reciprocal(Prim.a1,Prim.a2,Prim.a3);
Four.g2 = Reciprocal(Prim.a2,Prim.a3,Prim.a1);
Four.g3 = Reciprocal(Prim.a3,Prim.a1,Prim.a2);

Vg = Four.g1*cross(Four.g2,Four.g3)'; % reciprocal volume
Four.Radius = (3*Vg*Four.nGMax/(4*pi))^(1/3); % Sphere.Radius for nG reciprocal vectors
clear Vg

%% Reziprocal Space: fourier coefficients

Four.fG = fftn(Real.A)/numel(Real.A); % Fourier coefficients
[Four.xx,Four.yy,Four.zz] = Space(Four.g1,Four.g2,Four.g3,...
    getnk(Real.na1),getnk(Real.na2),getnk(Real.na3));

IDC = find(getR2(Four,[0 0 0])<Four.Radius^2); % indizes for Gvec in Four.G
Four.GC = [Four.xx(IDC),Four.yy(IDC),Four.zz(IDC)]; % Lattice points

% We rearrange these Fourier Coefficients for the matrix for the
% eigenvalue problem. We sort the Coefficients regarding |G| in
% ascending order, makes it a bit more clear for programming...
[~,sortID] = sort(sum(Four.GC.^2,2)); % id rearranging
IDC = IDC(sortID); % IDC sorted...
Four.GC = [Four.xx(IDC),Four.yy(IDC),Four.zz(IDC)];
Four.fGCi = Four.fG(IDC); % f(G used)
Four.fGC = zeros(size(Four.fG)); Four.fGC(IDC) = Four.fGCi;
Four.IDC = IDC;
fprintf('<strong>%d G-vectors</strong> are used (%d*%d-matrix)\n',numel(IDC)*[1 2 2])
clear IDC sortID

%% Reziprocal Space: symmetric points
% Symmetric points
name = fieldnames(Four.Points);
Four.Symmetric = struct('Name',[],'Points',[]);
Four.Symmetric.Name = name;
Four.Symmetric.Points = NaN(numel(name),3);
for k = 1:numel(name)
    Four.Symmetric.Points(k,:) = Four.Points.(name{k});
    Four.Symmetric.Name{k} = strrep(Four.Symmetric.Name{k},'G','\Gamma');
end
Four.Symmetric.Points = FromSpace(Four.g1,Four.g2,Four.g3,Four.Symmetric.Points);
Four = rmfield(Four,'Points');
clear name

fprintf('available points: <strong>%s</strong>\n',strrep([Four.Symmetric.Name{:}],'\Gamma','G'))
if isfield(Cell,'Trace')
    Four.Trace = Cell.Trace;
    Cell = rmfield(Cell,'Trace');
end

name = Four.Trace;
Four.Trace = struct('Name',[],'Points',[]);
Four.Trace.Name = cell(numel(name,1));
Four.Trace.Edges = NaN(numel(name),3);
for k = 1:numel(name)
    Four.Trace.Name{k} = strrep(name(k),'G','\Gamma');
    IDP = strcmp(Four.Symmetric.Name,Four.Trace.Name{k});
    if ~sum(IDP)
        error('Point "%s" of Trace not found in Symmetric Points (%s)',Four.Trace.Name{k},Cell.Bravais)
    end
    Four.Trace.Edges(k,:) = Four.Symmetric.Points(IDP,:);
end
clear name

Four.Trace.s = [0;cumsum(vecnorm(diff(Four.Trace.Edges,1,1)))];
nSegAdd = Four.nDispersion-numel(Four.Trace.s);
Four.Trace.ss = sort([Four.Trace.s;Four.Trace.s(end)*linspace(0,1,nSegAdd)']);
Four.Trace.Points = interp1(Four.Trace.s,Four.Trace.Edges,Four.Trace.ss);
clear IDP nSegAdd sortID

%% Real Space: ifft(Reciprocal Space)
Real.A0 = abs(ifftn(numel(Four.fGC)*Four.fGC));

%% Real Space: plotting %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if Calc.plotReal
    map = struct('xx',[],'yy',[],'zz',[],'up',[],'do',[]);
    r = [0 1 0; 1 0 0; 0 0 1]; % direction of mapping
    rotedges = [0 pi/2 -pi/2];
    xvzedges = [1 1 -1];
    yvzedges = [-1 1 1];
    rotmap = [0 0 -pi/2];
    xvzmap = [1 1 -1];
    yvzmap = [1 1 1];
    IDfig = [1 2 4];
    % r = [0 0 -1]
    xl = [inf -inf];
    yl = [inf -inf];
    for k = 1:size(r,1)
        [map(k).xx,map(k).yy,map(k).zz] = ...
            DensityMap(Real.xx, Real.yy, Real.zz, abs(Real.A0), r(k,:), 0.9*max([Real.na1,Real.na2,Real.na3]));
        [map(k).up,map(k).do] = ...
            Edges(Real.xx, Real.yy, Real.zz, r(k,:));
        map(k).up(:,end) = max(map(k).zz(:))+2;
        map(k).do(:,end) = max(map(k).zz(:))+1;
        if rotedges(k)
            R2 = [cos(rotedges(k)) -sin(rotedges(k));sin(rotedges(k)) cos(rotedges(k))];
            map(k).up(:,1:2) = map(k).up(:,1:2)*R2;
            map(k).do(:,1:2) = map(k).do(:,1:2)*R2;
            clear xx
        end
        if rotmap(k)
            R2 = [cos(rotmap(k)) -sin(rotmap(k));sin(rotmap(k)) cos(rotmap(k))];
                   xx = R2(1,1)*map(k).xx + R2(2,1)*map(k).yy;
            map(k).yy = R2(1,2)*map(k).xx + R2(2,2)*map(k).yy;
            map(k).xx = xx;
            clear xx
        end
        if xvzmap(k) == -1
            map(k).xx = -map(k).xx;
        end
        if yvzmap(k) == -1
            map(k).yy = -map(k).yy;
        end
        if xvzedges(k) == -1
            map(k).up(:,1) = -map(k).up(:,1);
            map(k).do(:,1) = -map(k).do(:,1);
        end
        if yvzedges(k) == -1
            map(k).up(:,2) = -map(k).up(:,2);
            map(k).do(:,2) = -map(k).do(:,2);
        end
        xl(1) = min(min(map(k).up(:,1)),xl(1));
        xl(2) = max(max(map(k).up(:,1)),xl(2));
        yl(1) = min(min(map(k).up(:,2)),yl(1));
        yl(2) = max(max(map(k).up(:,2)),yl(2));
    end
    xl = xl+0.02*[-1 1]*(max(xl)-min(xl));
    yl = yl+0.02*[-1 1]*(max(yl)-min(yl));
    
    
    figure('OuterPosition',P.rectp)
    for k = 1:numel(map)
        subplot(2,2,IDfig(k))
        hold on
        surf(1e9*map(k).xx,1e9*map(k).yy,1e9*map(k).zz,'LineStyle','none') % ,'FaceColor','interp')
        plot3(1e9*map(k).up(:,1),1e9*map(k).up(:,2),1e9*map(k).up(:,3),'Color',[0.3 0.3 0.8],'LineWidth',2)
        plot3(1e9*map(k).do(:,1),1e9*map(k).do(:,2),1e9*map(k).do(:,3),'--','Color',[0.3 0.3 0.8],'LineWidth',1.6)
        axis equal
        view(0,90)
        colormap('copper')
        set(gca,'Color',P.CBG)
        set(gca,'XColor',P.CAX,'FontWeight','bold')
        set(gca,'YColor',P.CAX,'FontWeight','bold')
        set(gca,'TickDir','out')
        xlim(1e9*xl)
        ylim(1e9*yl)
        if IDfig(k) == 1
            set(gca, 'YAxisLocation', 'right')
            set(gca,'Xdir','reverse')
            xlabel('x [nm]')
            ylabel('z [nm]')
            title('x-z','Color',P.CAX)
        elseif IDfig(k) == 2
            xlabel('y [nm]')
            title('y-z','Color',P.CAX)
        elseif IDfig(k) == 4
            set(gca, 'XAxisLocation', 'top')
            set(gca,'Ydir','reverse')
            ylabel('x [nm]')
            title('x-y','Color',P.CAX)
        end
    end
    
    subplot(2,2,3) % 3d image
    [~,~,xx,yy,zz] = ...
        Edges(Real.xx, Real.yy, Real.zz, [0 0 1]);
    patch(1e9*xx,1e9*yy,1e9*zz,[0.4 0.4 0.6],'LineWidth',1.6,'FaceAlpha',0.5,'EdgeColor',[0.3 0.3 0.8])
    hold on
    [xx,yy,zz] = Space(Prim.a1,Prim.a2,Prim.a3,0:1);
    Rmax = 0;
    for j = 1:numel(Sphere)
        [x,y,z] = sphere(20);
        x = Sphere(j).Radius*x+Sphere(j).M(1);
        y = Sphere(j).Radius*y+Sphere(j).M(2);
        z = Sphere(j).Radius*z+Sphere(j).M(3);
        for k = 1:numel(xx)
            surf(1e9*(x+xx(k)),1e9*(y+yy(k)),1e9*(z+zz(k)),'FaceColor',[0.4 0.8 0.4],'EdgeColor','none')
        end
        Rmax = max(Rmax,Sphere(j).Radius);
    end
    axis equal
    xlim(1e9*[min(xx(:))-Rmax,max(xx(:))+Rmax])
    ylim(1e9*[min(yy(:))-Rmax,max(yy(:))+Rmax])
    zlim(1e9*[min(zz(:))-Rmax,max(zz(:))+Rmax])
    set(gca,'Color',P.CBG)
   	set(gca,'XColor',P.CAX,'FontWeight','bold')
    set(gca,'YColor',P.CAX,'FontWeight','bold')
    set(gca,'ZColor',P.CAX,'FontWeight','bold')
    grid off
    xlabel('x [nm]')
    ylabel('y [nm]')
    zlabel('z [nm]')
    title('3D','Color',P.CAX)
    light
    lighting phong
    set(gcf,'Color',P.CBG)
    view(35,30)
    
    h = suptitle(sprintf('%s: density map of primitiv unit cell\nifft3 of %d fourier coefficients\n%s, %d spheres, %dx%dx%d voxels',Calc.name,numel(Four.IDC),Cell.Bravais,numel(Sphere),Real.na1,Real.na2,Real.na3));
    h.Color = P.CAX;
    h.FontSize = 12;
    h.FontWeight = 'bold';
    clear xl yl r R2 xx yy zz xvz yvz IDfig rot x y z Rmax map
    scf('jpg','UnitCell')
end

%% Reziprocal Space: plotting %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if Calc.plotFour
    figure('OuterPosition',P.rectp)
    hold on
    plot3([0;Four.g1(1)],[0;Four.g1(2)],[0;Four.g1(3)],'LineWidth',2,'Color',[0.6 0.6 0.8])
    plot3([0;Four.g2(1)],[0;Four.g2(2)],[0;Four.g2(3)],'LineWidth',2,'Color',[0.1 0.8 0.6])
    plot3([0;Four.g3(1)],[0;Four.g3(2)],[0;Four.g3(3)],'LineWidth',2,'Color',[0.8 0.0 0.8])
    plot3(Four.Trace.Edges(:,1),Four.Trace.Edges(:,2),Four.Trace.Edges(:,3),'-','LineWidth',1.2,'Color',[0.8 0.2 0.2])
    plot3(Four.Symmetric.Points(:,1),Four.Symmetric.Points(:,2),Four.Symmetric.Points(:,3),'o','LineWidth',1.6,'Color',[0.9 0.1 0.1])
    scatter3(Four.GC(:,1),Four.GC(:,2),Four.GC(:,3),140*max(abs(Four.fGCi),0.001)/max(abs(Four.fGCi)),[0.2 0.2 0.6],'LineWidth',1.6,'MarkerFaceColor','w')
    [xx,yy,zz] = WignerSeitz(Four.g1,Four.g2,Four.g3);
    patch(xx,yy,zz,(1:size(xx,2))','LineWidth',1.6,'FaceAlpha',0.3)
    clear xx yy zz
    legend(...
        sprintf('g1 = %1.1e',vecnorm(Four.g1)),...
        sprintf('g2 = %1.1e',vecnorm(Four.g2)),...
        sprintf('g3 = %1.1e',vecnorm(Four.g3)),...
        'Trace',...
        'Symmetric points',...
        'Reziprocal lattice points',...
        'First brillouin zone',...
        'Location','SouthOutside')

    for k = 1:numel(Four.Symmetric.Name) % display point name
        name = ['  \leftarrow',Four.Symmetric.Name{k}];
        text(Four.Symmetric.Points(k,1),Four.Symmetric.Points(k,2),Four.Symmetric.Points(k,3),name)
    end
    clear name

    axis equal
    set(gcf,'Color','w')
    set(gca,'clipping','off')
    view(40,25)
    title(sprintf('%s: Fourier coefficients\n%s, %d coefficients used',Calc.name,Cell.Bravais,numel(Four.IDC)))
    xlabel('kx [2\pi/nm]')
    ylabel('ky [2\pi/nm]')
    zlabel('kz [2\pi/nm]')
    scf('jpg','ReziprocalLattice')
end

%% Construct Eigenvalue problem
G = Four.GC; % G-vectors
N = size(G,1); % number of G-vectors -> Theta = hermitian 2Nx2N matrice
[X,Y,Z] = Space([1 0 0],[0 1 0],[0 0 1],... % Indices in x,y,z for Four.fG
    1:Real.na1,1:Real.na2,1:Real.na3);
G1 = (1:N)'*ones(1,N); % indices for G1
G2 = G1'; % indices for G2
G1 = G1(:); % 1d vector
G2 = G2(:); % 1d vector
XYZ = [X(Four.IDC(G1)),Y(Four.IDC(G1)),Z(Four.IDC(G1))] - ... % indices(d1,d2,d3)
    [X(Four.IDC(G2)),Y(Four.IDC(G2)),Z(Four.IDC(G2))];
XYZ(:,1) = mod(XYZ(:,1),Real.na1); % closed boundary condition
XYZ(:,2) = mod(XYZ(:,2),Real.na2); % closed boundary condition
XYZ(:,3) = mod(XYZ(:,3),Real.na3); % closed boundary condition
ID12 = XYZ(:,1)+Real.na1*XYZ(:,2)+Real.na1*Real.na2*XYZ(:,3)+1; % inices in Four.fG for f(G1-G2)
f12 = reshape(Four.fG(ID12),N,N); % f(G1+N*(G2-1)), see Theta
clear G1 G2 XYZ X Y Z ID12

L1 = mod((1:2*N)'-1,2)*ones(1,2*N); % Lambda1 (polarization 1)
G1 = floor(1:0.5:N+0.5)'*ones(1,2*N); % indices G1
L2 = L1'; % Lambda2 (polarization 2)
G2 = G1'; % indices G2
L1 = L1(:); % 1d vector
G1 = G1(:); % 1d vector
L2 = L2(:); % 1d vector
G2 = G2(:); % 1d vector
k = [1,2,3]; % dummy, testvector
ek = polvec(k+G); % Allocation
Th = Theta(k); % Allocation
% control if hermitian:
if sum(sum(Th-Th'~=0))
    warning('Matrix Theta not hermitian')
end
clear k Th

%% Dispersion relation: get Eigenvalues
nEig = Four.nDispersion;
Four.Eig = ones(nEig,2*N);
clrstr = '';
fprintf('Eigenvalues: ')
tic
sqrtEig2w = 2*Cell.a/(2*pi); % w*a/(2*pi*c); 2 because of two polarizations
for j = 1:nEig
    if ~mod(j,ceil(nEig/20)) || j == nEig
        str = sprintf('%1.0f/%1.0f\n',j,Four.nDispersion);
        fprintf([clrstr,str]);
        clrstr = repmat('\b',1,numel(str));
    end
    k = Four.Trace.Points(j,:);
    Four.Eig(j,:) = eig(Theta(k));
end
fprintf('\n')
tDispersion = toc;
Four.w = sqrtEig2w*sort(abs(sqrt((Four.Eig ))),2);
clear nM M clrstr str n

%% Dispersion relation: plotting %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if Calc.plotDispersion
    if Calc.EmptyLatticeApproximation
        C = [0 0 0];
    else
        C = [0.1 0.1 0.8];
    end
    figure('OuterPosition',P.rectl)
    hold on
    if isempty(Calc.yLimDispersion)
        yl = [0,max(Four.w(:))];
        yl = [0 min(mean(Four.w(:)),yl(2))];
    else
        yl = Calc.yLimDispersion;
    end
    yy = [yl';NaN]*ones(1,numel(Four.Trace.s));
    xx = [1;1;NaN]*Four.Trace.s';
    plot(xx(:),yy(:),'LineWidth',1.6,'Color',[0.6 0.6 0.6])
    clear xx yy
    plot(Four.Trace.ss,Four.w,'-','Color',C,'LineWidth',1.8)
    set(gcf,'Color','w')
    set(gca,'XTick',Four.Trace.s,'XTickLabel',Four.Trace.Name)
    ylim(yl)
    xlim([0,Four.Trace.s(end)])
    ylabel('frequency \omegaa/2\pic')
    title(sprintf('%s: photonic crystal dispersion relation\n%s, %d spheres, %d G-vectors, %dx%d-matrix',Calc.name,Cell.Bravais,numel(Sphere),numel(Four.IDC)*[1 2 2]))
    scf('pdf','DispersionRelation')
end

%% Density of states: get Eigenvalues
if Calc.plotDOS
    nAx = 2*floor(Four.nDOS/2)+1; % nEig odd number; avoid symmetries
    estimatedTime = tDispersion*nAx^3/nEig;
    fprintf('estimated time for DOS: <strong>%1.1fmin</strong>\n',1/60*estimatedTime)
    sm('start',estimatedTime)
    [kx,ky,kz] = Space(Four.g1,Four.g2,Four.g3,(0:nAx-1)/nAx,(0:nAx-1)/nAx,(0:nAx-1)/nAx);
    kx = kx+0.25*vecnorm(Four.g1)/nAx; % avoid symmetries
    ky = ky+0.25*vecnorm(Four.g2)/nAx; % avoid symmetries
    kz = kz+0.25*vecnorm(Four.g3)/nAx; % avoid symmetries

    % wFaktor: result Eig = w^2/c^2
    % -> w = c/(sqrt(e))*sqrt(Eig); % c/sqrt(e) speed of light in cell medium
    sqrtEig2w = 2*3e8; % w*a/(2*pi*c); 2 because of two polarizations
    % from Integral( w^2/(pi^2*c^3) ); w from 0 to 2*pi*c/L (= w at first boundary)
    % -> Integral == 2 (2 modes in 0...2pi/L)
    factorDOS = Cell.e^(3/2) * 8*pi/(3*Prim.Volume);
    
    Eig3 = ones(nAx,nAx,2*N); % Block (of which nAx have to be calculated)
    nDOSAxis = ceil(0.5*nAx^1.55*N^0.5); % fit
    clrstr = '';
    fprintf('Eigenvalues: ')
    tic
    for j1 = 1:nAx
        str = sprintf('%1.0f/%1.0f\n',j1,nAx);
        fprintf([clrstr,str]);
        clrstr = repmat('\b',1,numel(str));
        for j2 = 1:nAx
            for j3 = 1:nAx % Eigenvalues at kpoint(j1,j2,j3)
                Eig3(j2,j3,:) = eig(Theta([kx(j1,j2,j3),ky(j1,j2,j3),kz(j1,j2,j3)]));
            end
        end
        if j1 == 1
            wDOS = linspace(0,2*sqrtEig2w*max(abs(sqrt(Eig3(:)))),nDOSAxis)';
            dwDOS = wDOS(2)-wDOS(1);
            DOS = zeros(nDOSAxis,1);
        end
        DOS = LinearMapping(DOS,sqrtEig2w*abs(sqrt(Eig3(:))),dwDOS,nDOSAxis);
    end
    DOS = factorDOS*DOS;
    DOS = DOS/(nAx^3*dwDOS);
    DOS = ([0;DOS(2:end)]+[DOS(2:end);0])/2;
    
    ID = find(DOS>0,1,'last');
    DOS = DOS(1:ID);
    wDOS = wDOS(1:ID);
    fprintf('\n')
    tDOS = toc;
    Four.w = Cell.a*sort(abs(sqrt((Four.Eig ))),2)/(2*pi).^2; % w*a/(2*pi*c)
    clear nM M clrstr str j1 j2 j3 Eig3 wFaktor ID
    clear ek f12 G G1 G2 L1 L2 N 
    fprintf('calculation time for DOS: <strong>%1.1fmin</strong>\n',1/60*tDOS)
end

%% Density of states: plotting and saving %%%%%%%%%%%%%%%%%%%%%%%%%%
if Calc.save && Calc.plotDOS
    save([P.SavePathFigure,'\DOS.mat'],'DOS','wDOS')
end
if Calc.plotDOS
    figure('OuterPosition',P.rectl)
    hold on
    C = [0.1 0.1 0.8];
    
    xl = find(cumsum(DOS)*dwDOS>12*factorDOS,1,'first'); % collect 12 modes...
    if isempty(xl); xl = [0,wDOS(ceil(end/4))+1]; else; xl = [0,wDOS(xl)+1]; end
    yl = [0 1.2*max(DOS(wDOS<xl(2)))];
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     DOS0 = wDOS.^2/(pi^2*(3e8)^3); % empty lattice DOS
% 	plot(wDOS,DOS0,'LineWidth',2,'Color',[0.75 0.75 0.75])
% 	hold on
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	plot(wDOS,DOS,'LineWidth',1.6,'Color',C)
    set(gcf,'Color','w')
    xlim(xl)
    ylim(yl)
    xlabel('frequency [rad/s]')
    ylabel('D(\omega)')
    title(sprintf('%s: photonic crystal density of states\n%s, %d spheres, %d G-vectors\n%dx%d-matrix for %d^3 independent k-space points',Calc.name,Cell.Bravais,numel(Sphere),numel(Four.IDC)*[1 2 2],nAx))
    scf('pdf','DOS')
    sm('end',tDOS)
end

%% nested functions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function sm(type,t)
        if Calc.saveplots
            setpref('Internet','E_mail','bertram.d@gmx.at');
            mail = 'dumboeck@pjm.co.at';
            pwd = 'nucleon88';
            setpref('Internet','SMTP_Server','smtp.gmail.com');
            setpref('Internet','E_mail',mail);
            setpref('Internet','SMTP_Username',mail);
            setpref('Internet','SMTP_Password',pwd);
            props = java.lang.System.getProperties;
            props.setProperty('mail.smtp.auth','true');
            props.setProperty('mail.smtp.socketFactory.class', 'javax.net.ssl.SSLSocketFactory');
            props.setProperty('mail.smtp.socketFactory.port','465');

            switch type
                case 'start'
                    tl = sprintf('Photonic Crystal: "%s"; start calculating density of states',Calc.name);
                    msg = sprintf('start calculation: density of states for %s, estimated time = %1.1f hours',Calc.name,t/3600);
                    attachment = {[P.SavePathFigure,'\UnitCell.jpg'],[P.SavePathFigure,'\ReziprocalLattice.jpg'],[P.SavePathFigure,'\DispersionRelation.pdf']};
                case 'end'
                    tl = sprintf('Photonic Crystal: "%s"; end of calculation',Calc.name);
                    msg = sprintf('end calculation: density of states for %s, calculation time = %1.1f hours',Calc.name,t/3600);
                    attachment = {[P.SavePathFigure,'\DOS.pdf']};
                otherwise
                    error('type "%s" not supported')
            end
            sendmail('bertram.d@gmx.at',tl,msg,attachment);
        end
    end
    function scf(fmt,name)
        if Calc.saveplots
            if nargin < 2
                ax = gca;
                name = str2fn(ax.Title.String);
            end
            if nargin < 1
                fmt = 'pdf';
            end
            switch lower(fmt)
                case 'jpg'
                    mkjpg(gcf,P.SavePathFigure,name,400);
                case 'pdf'
                    mkpdf(gcf,P.SavePathFigure,name);
                otherwise
                    error(['fmt "',fmt,'" not supported'])
            end
        end
    end
    function Th = Theta(k)
        ek = polvec(k+G);
        Th = reshape(dot(dot(cross(k+G(G1,:),ek(G1+L1*N,:)), ...
            cross(k+G(G2,:),ek(G2+L2*N,:)), 2),...
            f12(G1+N*(G2-1)), 2),2*N,2*N);

    end
end

%% local functions
    function [Real,Prim,Cell] = BasisVectors(Real,Cell)
    % function [a1,a2,a3] = BasisVectors(a,b,c,al,be,ga,type)
        % converts triclinic basis vectors to kartesian koordinates
        a = Cell.a;
        b = Cell.b;
        c = Cell.c;
        al = pi*Cell.alpha/180;
        be = pi*Cell.beta/180;
        ga = pi*Cell.gamma/180;
        
        a1 = a*[1 0 0]; % align a1 with x-axis
        a2 = b*[cos(ga) sin(ga) 0]; % a1-a2 plane = x-y plane (pi1)
        % a3 is a bit tricky...
        a3x = cos(be); % scaled with 1/c
        a3y = (cos(al)-cos(ga)*cos(be))/sin(ga); % scaled with 1/c
        a3 = c*[a3x a3y sqrt(1-a3x^2-a3y^2)];
        Cell.a1 = a1;
        Cell.a2 = a2;
        Cell.a3 = a3;
        
        switch Cell.Type % convert to primitive unit cell
            case 'simple'
                % do nothing
            case 'base'
                a2 = (a1+a2)/2;
            case 'body'
                a3 = (a1+a2+a3)/2;
            case 'face'
                a1o = a1;
                a2o = a2;
                a3o = a3;
                a1 = (a1o+a2o)/2;
                a2 = (a2o+a3o)/2;
                a3 = (a3o+a1o)/2;
                clear a1o a2o a3o
            otherwise
                error (['Type "',Cell.Type,'" not supported'])
        end
        
        Prim.e = Cell.e;
        Prim.a = vecnorm(a1);
        Prim.b = vecnorm(a2);
        Prim.c = vecnorm(a3);
        Prim.alpha = vecangle(a2/Prim.b,a3/Prim.c);
        Prim.beta = vecangle(a3/Prim.c,a1/Prim.a);
        Prim.gamma = vecangle(a1/Prim.a,a2/Prim.b);
        Prim.a1 = a1;
        Prim.a2 = a2;
        Prim.a3 = a3;
    end
	function b1 = Reciprocal(a1,a2,a3)
        % reciprocal lattice vector
        b1 = 2*pi*cross(a2,a3)/(a1*cross(a2,a3)');
    end
    function R2 = getR2(data,M)
        R2 = (data.xx-M(1)).^2 + (data.yy-M(2)).^2 + (data.zz-M(3)).^2;
    end
    function nk = getnk(n)
        nk = [(1:ceil((n+1)/2))-1,-(floor((n+1)/2):-1:2)+1];
    end
    function y = LinearMapping(y,xi,dx,n)
        xi = xi/dx;
        a = xi-floor(xi);
        b = ceil(xi)-xi;
        xi = floor(xi);
        % xi(xi>n-1) = []; % should never happen...
        for k = 1:numel(xi) % linear interpolation
            ID = xi(k)+1;
            y(ID) = y(ID) + b(k);
            y(ID+1) = y(ID+1) + a(k);
        end
    end
    